"""Library functions for YamlTable."""


from typing import Any, Iterable, List

import yaml
from yamltable.typing import Path, Row


def read(file_path: Path) -> List[Row]:
    """Read data from YAML file.

    :param file_path: YAML file path
    :return: YAML data
    :raise TypeError: if file is not organized as a list
    """

    with open(file_path, "r") as handle:
        rows = yaml.safe_load(handle)

    if isinstance(rows, list):
        return rows
    else:
        raise TypeError("YAML file does not contain a list")


def search(key: str, val: Any, rows: Iterable[Row]) -> List[Row]:
    """Search dictionaries by key and value.

    :param key: search key
    :param val: key comparison value
    :param rows: dictionaries to search
    :return: matching dictionaries
    """

    return [row for row in rows if key in row and row[key] == val]


def sort(key: str, rows: Iterable[Row]) -> List[Row]:
    """Sort dictionaries based on value for supplied key name.

    :param key: search key
    :param rows: dictionaries to sort
    :return: list of sorted dictionaries
    """

    return sorted(rows, key=lambda row: row[key])


def write(file_path: Path, rows: List[Row], sort_keys: bool = False) -> None:
    """Write data to YAML file.

    Additional sorts dictionary keys.

    :param file_path: YAML file path
    :param rows: list of dictionaries to write
    """

    with open(file_path, "w") as handle:
        yaml.dump(rows, handle, sort_keys=sort_keys)



"""Command line interface for YamlTable."""


import argparse
import pprint

import yamltable


def main() -> None:
    """Command line interface for YAML sorter."""

    parser = argparse.ArgumentParser(
        description="utilities for for working with list organized YAML files"
    )
    subparser = parser.add_subparsers(dest="command", required=True)

    list_parser = subparser.add_parser(name="list", description="list dictionary key values")
    list_parser.add_argument("key", type=str, help="dictionary key")
    list_parser.add_argument("file_path", type=str, help="YAML file location")

    search_parser = subparser.add_parser(
        name="search", description="search dictionaries by key and value"
    )
    search_parser.add_argument("key", type=str, help="dictionary key")
    search_parser.add_argument("value", help="key value")
    search_parser.add_argument("file_path", type=str, help="YAML file location")

    sort_parser = subparser.add_parser(
        name="sort", description="sort dictionaries by key and value"
    )
    sort_parser.add_argument("key", type=str, help="dictionary key")
    sort_parser.add_argument("file_path", type=str, help="YAML file location")

    args = parser.parse_args()
    dicts = yamltable.read(args.file_path)

    if args.command == "list":
        for dict_ in dicts:
            print(dict_[args.key])
    elif args.command == "search":
        for match in yamltable.search(args.key, args.value, dicts):
            pprint.pprint(match)
    elif args.command == "sort":
        sorted_dicts = yamltable.sort(args.key, dicts)
        yamltable.write(args.file_path, sorted_dicts)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['yamltable']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml>=5.2,<6.0']

entry_points = \
{'console_scripts': ['yamltable = yamltable.main:main']}

setup_kwargs = {
    'name': 'yamltable',
    'version': '0.0.1',
    'description': 'Command line utility for list organized YAML files.',
    'long_description': '# YamlTable \n\n![](https://github.com/wolfgangwazzlestrauss/yamltable/workflows/build/badge.svg)\n\nYamlTable is a Python command line utility for working with YAML files organized similar to a\nrelational database table. It supports YAML files organized as a list of dictionaries, which share\nkey names and value types. YamlTable provides commands for listing, searching, sorting, etc. data\nfrom the supported files.\n\n\n## Getting Started\n\n### Installation\n\nYamlTable can be installed for Python 3.7+ with `pip`.\n\n```bash\npip install --user yamltable\n```\n\n### Commands\n\n\n\n## Contributing\n\nSince YamlTable is in an early development phase, it is not currently open to contributors.\n\n\n## License\n\nLicensed under the [MIT](license.txt) license.\n\n',
    'author': 'Macklan Weinstein',
    'author_email': 'wolfgangwazzlestrauss@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/wolfgangwazzlestrauss/yamltable',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
